@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,7 +16,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
 import math
@@ -52,7 +52,6 @@ class Serial(AbstractEdge):
         # get mini-vector, and its angle
         rot_angle = - math.pi / 4.0
         vectrot = QtCore.QPointF(math.cos(vector_angle + rot_angle), math.sin(vector_angle + rot_angle))
-        vectrot_angle = math.atan2(vectrot.y(), vectrot.x())
 
         # get the rotated points position
         angle_srcPt = QtCore.QPointF(self.src.x() + self.dx / 2.0 + 15 * vectrot.x(), self.src.y() + self.dy / 2.0 + 15 * vectrot.y())
@@ -95,7 +94,7 @@ class Serial(AbstractEdge):
 
             # if nodes are too close, points disappears
             if self.length < 80:
-               return
+                return
 
             # source point
             if self.src_interface_status == 'up':
@@ -109,18 +108,30 @@ class Serial(AbstractEdge):
             
             if globals.GApp.workspace.flg_showInterfaceNames:
                 if self.labelSouceIf == None:
-                    self.labelSouceIf = Annotation()
-                    self.labelSouceIf.setPlainText(self.srcIf)
-                    self.labelSouceIf.setPos(self.src)
-                    self.labelSouceIf.setZValue(2)
-                    globals.GApp.topology.addItem(self.labelSouceIf)
-                    self.labelSouceIf.autoGenerated = True
+
+                    self.labelSouceIf = Annotation(self.source)
+
+                    if globals.interfaceLabels.has_key(self.source.hostname + ' ' + self.srcIf):
+                        annotation = globals.interfaceLabels[self.source.hostname + ' ' + self.srcIf]
+                        self.labelSouceIf.setZValue(annotation.zValue())
+                        self.labelSouceIf.setDefaultTextColor(annotation.defaultTextColor())
+                        self.labelSouceIf.setFont(annotation.font())
+                        self.labelSouceIf.setPlainText(annotation.toPlainText())
+                        self.labelSouceIf.setPos(annotation.x(), annotation.y())
+                        self.labelSouceIf.rotation = annotation.rotation
+                        self.labelSouceIf.rotate(annotation.rotation)
+                        del globals.interfaceLabels[self.source.hostname + ' ' + self.srcIf]
+                    else:
+                        self.labelSouceIf.setPlainText(self.srcIf)
+                        self.labelSouceIf.setPos(self.mapToItem(self.source, self.src))
+                        self.labelSouceIf.autoGenerated = True
+                    self.labelSouceIf.deviceName = self.source.hostname
+                    self.labelSouceIf.deviceIf = self.srcIf
                 if not self.labelSouceIf.isVisible():
-                    self.labelSouceIf.setPos(self.src)
                     self.labelSouceIf.show()
             elif self.labelSouceIf and globals.GApp.workspace.flg_showInterfaceNames == False:
                 self.labelSouceIf.hide()
-            
+
             painter.drawPoint(self.src)
 
             # destination point
@@ -135,14 +146,26 @@ class Serial(AbstractEdge):
             
             if globals.GApp.workspace.flg_showInterfaceNames:
                 if self.labelDestIf  == None:
-                    self.labelDestIf = Annotation()
-                    self.labelDestIf.setPlainText(self.destIf)
-                    self.labelDestIf.setPos(self.dst)
-                    self.labelDestIf.setZValue(2)
-                    self.labelDestIf.autoGenerated = True
-                    globals.GApp.topology.addItem(self.labelDestIf)
+
+                    self.labelDestIf = Annotation(self.dest)
+
+                    if globals.interfaceLabels.has_key(self.dest.hostname + ' ' + self.destIf):
+                        annotation = globals.interfaceLabels[self.dest.hostname + ' ' + self.destIf]
+                        self.labelDestIf.setZValue(annotation.zValue())
+                        self.labelDestIf.setDefaultTextColor(annotation.defaultTextColor())
+                        self.labelDestIf.setFont(annotation.font())
+                        self.labelDestIf.setPlainText(annotation.toPlainText())
+                        self.labelDestIf.setPos(annotation.x(), annotation.y())
+                        self.labelDestIf.rotation = annotation.rotation
+                        self.labelDestIf.rotate(annotation.rotation)
+                        del globals.interfaceLabels[self.dest.hostname + ' ' + self.destIf]
+                    else:
+                        self.labelDestIf.setPlainText(self.destIf)
+                        self.labelDestIf.setPos(self.mapToItem(self.dest, self.dst))
+                        self.labelDestIf.autoGenerated = True
+                    self.labelDestIf.deviceName = self.dest.hostname
+                    self.labelDestIf.deviceIf = self.destIf
                 if not self.labelDestIf.isVisible():
-                    self.labelDestIf.setPos(self.dst)
                     self.labelDestIf.show()
             elif self.labelDestIf and globals.GApp.workspace.flg_showInterfaceNames == False:
                 self.labelDestIf.hide()
