@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007-2008 GNS3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,30 +16,32 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import os, re, random, time, base64, traceback
+import os, re, random, base64, traceback
 import GNS3.Globals as globals
 import GNS3.Dynagen.dynagen as dynagen_namespace
 import GNS3.Dynagen.dynamips_lib as lib
-import GNS3.Dynagen.pemu_lib as pix
+import GNS3.Dynagen.qemu_lib as qlib
 from GNS3.Globals.Symbols import SYMBOLS
-from GNS3.Utils import translate, debug, error, relpath
+from GNS3.Utils import translate, debug, error
 from PyQt4 import QtGui, QtCore
 from GNS3.Annotation import Annotation
 from GNS3.Pixmap import Pixmap
-from GNS3.HypervisorManager import HypervisorManager
+from GNS3.ShapeItem import AbstractShapeItem
+from GNS3.ShapeItem import Rectangle
+from GNS3.ShapeItem import Ellipse
 from GNS3.Config.Objects import iosImageConf, hypervisorConf
 from GNS3.Node.AbstractNode import AbstractNode
 from GNS3.Node.DecorativeNode import DecorativeNode, init_decoration_id
 from GNS3.Node.IOSRouter import IOSRouter, init_router_id
-from GNS3.Node.ATMSW import ATMSW, init_atmsw_id
-from GNS3.Node.ATMBR import ATMBR, init_atmbr_id
+from GNS3.Node.ATMSW import init_atmsw_id
+from GNS3.Node.ATMBR import init_atmbr_id
 from GNS3.Node.ETHSW import ETHSW, init_ethsw_id
-from GNS3.Node.FRSW import FRSW, init_frsw_id
+from GNS3.Node.FRSW import init_frsw_id
 from GNS3.Node.Cloud import Cloud, init_cloud_id
-from GNS3.Node.FW import FW, init_fw_id
+from GNS3.Node.AnyEmuDevice import init_emu_id, AnyEmuDevice
 
 router_hostname_re = re.compile(r"""^R([0-9]+)""")
 ethsw_hostname_re = re.compile(r"""^SW([0-9]+)""")
@@ -47,7 +49,7 @@ frsw_hostname_re = re.compile(r"""^FR([0-9]+)""")
 atmsw_hostname_re = re.compile(r"""^ATM([0-9]+)""")
 atmbr_hostname_re = re.compile(r"""^BR([0-9]+)""")
 cloud_hostname_re = re.compile(r"""^C([0-9]+)""")
-firewall_hostname_re = re.compile(r"""^FW([0-9]+)""")
+emu_hostname_re = re.compile(r"""^[FW|JUNOS|ASA|IDS|QEMU]([0-9]+)""")
 decorative_hostname_re = re.compile(r"""^N([0-9]+)""")
 
 class NETFile(object):
@@ -103,17 +105,17 @@ class NETFile(object):
                                                                                                                                                                             connection_list)
                                 elif isinstance(remote_device, lib.FRSW) or isinstance(remote_device, lib.ATMSW) or isinstance(remote_device, lib.ETHSW) or isinstance(remote_device, lib.ATMBR):
                                     connection_list.append((device.name, source_interface, remote_device.name, str(remote_port)))
-                                elif isinstance(remote_device, pix.FW):
+                                elif isinstance(remote_device, qlib.AnyEmuDevice):
                                     connection_list.append((device.name, source_interface, remote_device.name, remote_adapter + str(remote_port)))
 
-    def populate_connection_list_for_fw(self, device, connection_list):
-        """ Add firewall connections in connection_list
+    def populate_connection_list_for_emulated_device(self, device, connection_list):
+        """ Add emulated device connections in connection_list
         """
 
         for port in device.nios:
             if device.nios[port] != None:
                 (remote_device, remote_adapter, remote_port) = lib.get_reverse_udp_nio(device.nios[port])
-                if isinstance(remote_device, pix.FW):
+                if isinstance(remote_device, qlib.AnyEmuDevice):
                     self.add_in_connection_list((device.name, 'e' + str(port), remote_device.name, remote_adapter + str(remote_port)), connection_list)
                 elif isinstance(remote_device, lib.ETHSW):
                     connection_list.append((device.name, 'e' + str(port), remote_device.name, str(remote_port)))
@@ -122,35 +124,32 @@ class NETFile(object):
         """ Create a new node
         """
 
-        symbol_name = x = y = hx = hy = None
-        if isinstance(device, pix.FW) and  self.dynagen.globalconfig['pemu ' + device.dynamips.host].has_key(running_config_name):
-            pemu_config = self.dynagen.globalconfig['pemu ' + device.dynamips.host][running_config_name]
-            if pemu_config.has_key('x'):
-                x = pemu_config['x']
-            if pemu_config.has_key('y'):
-                y = pemu_config['y']
-            if pemu_config.has_key('hx'):
-                hx = pemu_config['hx']
-            if pemu_config.has_key('hy'):
-                hy = pemu_config['hy']
-            if pemu_config.has_key('symbol'):
-                symbol_name = pemu_config['symbol']
+        symbol_name = x = y = z = hx = hy = None
+        config = None
+        if isinstance(device, qlib.AnyEmuDevice) and self.dynagen.globalconfig['qemu ' + device.dynamips.host +':' + str(device.dynamips.port)].has_key(running_config_name):
+            config = self.dynagen.globalconfig['qemu ' + device.dynamips.host +':' + str(device.dynamips.port)][running_config_name]
         elif self.dynagen.globalconfig[device.dynamips.host +':' + str(device.dynamips.port)].has_key(running_config_name):
-            dynamips_config = self.dynagen.globalconfig[device.dynamips.host +':' + str(device.dynamips.port)][running_config_name]
-            if dynamips_config.has_key('x'):
-                x = dynamips_config['x']
-            if dynamips_config.has_key('y'):
-                y = dynamips_config['y']
-            if dynamips_config.has_key('hx'):
-                hx = dynamips_config['hx']
-            if dynamips_config.has_key('hy'):
-                hy = dynamips_config['hy']
-            if dynamips_config.has_key('symbol'):
-                symbol_name = dynamips_config['symbol']
+            config = self.dynagen.globalconfig[device.dynamips.host +':' + str(device.dynamips.port)][running_config_name]
+
+        if config:
+            if config.has_key('x'):
+                x = config['x']
+            if config.has_key('y'):
+                y = config['y']
+            if config.has_key('z'):
+                z = config['z']
+            if config.has_key('hx'):
+                hx = config['hx']
+            if config.has_key('hy'):
+                hy = config['hy']
+            if config.has_key('symbol'):
+                symbol_name = config['symbol']
 
         # symbol name not found, use default one
+        default_symbol = False
         if not symbol_name or not globals.GApp.scene.renders.has_key(symbol_name):
             symbol_name = default_symbol_name
+            default_symbol = True
 
         for item in SYMBOLS:
             if item['name'] == symbol_name:
@@ -158,11 +157,15 @@ class NETFile(object):
                 node = item['object'](renders['normal'], renders['selected'])
                 node.set_hostname(device.name)
                 node.type = item['name']
+                if not default_symbol:
+                    node.default_symbol = False
                 if x == None:
                     x = random.uniform(-200, 200)
                 if y == None:
                     y = random.uniform(-200, 200)
                 node.setPos(float(x), float(y))
+                if z:
+                    node.setZValue(float(z))
                 if hx and hy:
                     node.hostname_xpos = float(hx)
                     node.hostname_ypos = float(hy)
@@ -170,7 +173,7 @@ class NETFile(object):
                     node.showHostname()
                 debug("Node created: " + str(node))
                 return node
-        assert('SYMBOL NAME NOT FOUND: ' + symbol_name)
+    
         return None
 
     def record_image(self, device):
@@ -185,10 +188,10 @@ class NETFile(object):
         if device.model == 'c3725' or device.model == 'c3745':
             conf_image.platform = 'c3700'
         else:
-            conf_image.platform = device.model
-        conf_image.chassis = device.model_string
+            conf_image.platform = str(device.model)
+        conf_image.chassis = str(device.model_string)
         if device.idlepc:
-            conf_image.idlepc = device.idlepc
+            conf_image.idlepc = str(device.idlepc)
         conf_image.default_ram = device.ram
         conf_image.hypervisor_port = device.dynamips.port
         conf_image.default = False
@@ -250,12 +253,12 @@ class NETFile(object):
 
         globals.GApp.topology.recordLink(srcid, source_interface, dstid, destination_interface, src_node, dst_node)
 
-        if not isinstance(src_node, IOSRouter) and not isinstance(src_node, FW):
+        if not isinstance(src_node, IOSRouter) and not isinstance(src_node, AnyEmuDevice):
             if not isinstance(src_node,Cloud) and not src_node.hypervisor:
                 src_node.get_dynagen_device()
             src_node.startupInterfaces()
             src_node.state = 'running'
-        if not isinstance(dst_node, IOSRouter) and not isinstance(dst_node, FW):
+        if not isinstance(dst_node, IOSRouter) and not isinstance(dst_node, AnyEmuDevice):
             if not isinstance(dst_node,Cloud) and not dst_node.hypervisor:
                 dst_node.get_dynagen_device()
             dst_node.startupInterfaces()
@@ -293,6 +296,10 @@ class NETFile(object):
         max_decorative_id = -1
         gns3data = self.dynagen.getGNS3Data()
         if gns3data:
+            if  gns3data.has_key('width') and  gns3data.has_key('height'):
+                width = int(gns3data['width'])
+                height = int(gns3data['height'])
+                globals.GApp.topology.setSceneRect(-(width / 2), -(height / 2), width, height)
             if  gns3data.has_key('m11') and  gns3data.has_key('m22'):
                 globals.GApp.scene.setMatrix(QtGui.QMatrix(float(gns3data['m11']), 0.0,  0.0,  float(gns3data['m22']), 0.0,  0.0))
             for section in gns3data:
@@ -301,17 +308,23 @@ class NETFile(object):
                 except ValueError:
                     continue
                 if devtype.lower() == 'cloud':
+                    default_symbol = True
                     if gns3data[section].has_key('symbol') and globals.GApp.scene.renders.has_key(gns3data[section]['symbol']):
                         symbol = gns3data[section]['symbol']
+                        default_symbol = False
                     else:
                         symbol = 'Cloud'
                     renders = globals.GApp.scene.renders[symbol]
                     cloud = Cloud(renders['normal'], renders['selected'])
                     cloud.type = symbol
+                    if not default_symbol:
+                        cloud.default_symbol = False
                     cloud.hostname = unicode(hostname)
                     if gns3data[section].has_key('x') and gns3data[section].has_key('y') \
                         and gns3data[section]['x'] != None and gns3data[section]['y'] != None:
                         cloud.setPos(float(gns3data[section]['x']), float(gns3data[section]['y']))
+                    if gns3data[section].has_key('z'):
+                        cloud.setZValue(float(gns3data[section]['z']))
                     if gns3data[section].has_key('hx') and gns3data[section].has_key('hy') \
                         and gns3data[section]['hx'] != None and gns3data[section]['hy'] != None:
                         cloud.hostname_xpos = float(gns3data[section]['hx'])
@@ -338,12 +351,57 @@ class NETFile(object):
                             max_cloud_id = id
 
                 if devtype.lower() == 'note':
+
                     note_object = Annotation()
-                    note_object.setPlainText(unicode(gns3data[section]['text'].replace("\\n", "\n")))
+                    note_object.setPlainText(gns3data[section]['text'].replace("\\n", "\n"))
                     note_object.setPos(float(gns3data[section]['x']), float(gns3data[section]['y']))
                     if gns3data[section].has_key('z'):
                         note_object.setZValue(float(gns3data[section]['z']))
-                    globals.GApp.topology.addItem(note_object)
+                    if gns3data[section].has_key('font'):
+                        font = QtGui.QFont()
+                        if font.fromString(gns3data[section]['font'][1:-1]):
+                            note_object.setFont(font)
+                        else:
+                            print unicode(translate("NETFile", "Cannot load font: %s")) % gns3data[section]['font']
+                    if gns3data[section].has_key('rotate'):
+                        note_object.rotation = int(gns3data[section]['rotate'])
+                        note_object.rotate(note_object.rotation)
+                    if gns3data[section].has_key('color'):
+                        note_object.setDefaultTextColor(QtGui.QColor(gns3data[section]['color'][1:-1]))
+                        
+                    # this is an interface label, save it in a dict to be used later ...
+                    if gns3data[section].has_key('interface'):
+                        globals.interfaceLabels[gns3data[section]['interface']] = note_object
+                    else:
+                        globals.GApp.topology.addItem(note_object)
+                    
+                if devtype.lower() == 'shape':
+                    if gns3data[section]['type'] == 'rectangle':
+                        size = QtCore.QSizeF(float(gns3data[section]['width']), float(gns3data[section]['height']))
+                        pos = QtCore.QPointF(float(gns3data[section]['x']), float(gns3data[section]['y']))
+                        shape_object = Rectangle(pos, size)
+                    else:
+                        size = QtCore.QSizeF(float(gns3data[section]['width']), float(gns3data[section]['height']))
+                        pos = QtCore.QPointF(float(gns3data[section]['x']), float(gns3data[section]['y']))
+                        shape_object = Ellipse(pos, size)
+    
+                    if gns3data[section].has_key('z'):
+                        shape_object.setZValue(float(gns3data[section]['z']))
+                    if gns3data[section].has_key('rotate'):
+                        shape_object.rotation = int(gns3data[section]['rotate'])
+                        shape_object.rotate(shape_object.rotation)
+                    if gns3data[section].has_key('fill_color'):
+                        brush = QtGui.QBrush(QtGui.QColor(gns3data[section]['fill_color'][1:-1]))
+                        shape_object.setBrush(brush)
+                    pen = QtGui.QPen(QtCore.Qt.black, 2, QtCore.Qt.SolidLine, QtCore.Qt.RoundCap, QtCore.Qt.RoundJoin)
+                    if gns3data[section].has_key('border_color'):
+                        pen.setColor(QtGui.QColor(gns3data[section]['border_color'][1:-1]))
+                    if gns3data[section].has_key('border_width'):
+                        pen.setWidth(int(gns3data[section]['border_width']))
+                    if gns3data[section].has_key('border_style'):
+                        pen.setStyle(QtCore.Qt.PenStyle(int(gns3data[section]['border_style'])))
+                    shape_object.setPen(pen)
+                    globals.GApp.topology.addItem(shape_object)
                     
                 if devtype.lower() == 'pixmap':
                     pixmap_path = unicode(gns3data[section]['path'])
@@ -362,12 +420,15 @@ class NETFile(object):
                     hostname = unicode(hostname)
                     symbol = unicode(gns3data[section]['symbol'])
                     if not globals.GApp.scene.renders.has_key(symbol):
-                        print unicode(translate("NETFile", "%s: cannot find %s symbol")) % (hostname, symbol)
+                        print unicode(translate("NETFile", "%s: cannot find %s symbol, please check this symbol is in your node list and reload the .net file")) % (hostname, symbol)
                         continue
                     renders = globals.GApp.scene.renders[symbol]
                     decorative_node = DecorativeNode(renders['normal'], renders['selected'])
                     decorative_node.set_hostname(hostname)
+                    decorative_node.create_config()
                     decorative_node.setPos(float(gns3data[section]['x']), float(gns3data[section]['y']))
+                    if gns3data[section].has_key('z'):
+                        decorative_node.setZValue(float(gns3data[section]['z']))
                     decorative_node.type = symbol
                     QtCore.QObject.connect(decorative_node, QtCore.SIGNAL("Add link"), globals.GApp.scene.slotAddLink)
                     QtCore.QObject.connect(decorative_node, QtCore.SIGNAL("Delete link"), globals.GApp.scene.slotDeleteLink)
@@ -382,13 +443,12 @@ class NETFile(object):
                             max_decorative_id = id
                     if gns3data[section].has_key('connections'):
                         connections = gns3data[section]['connections'].split(' ')
-                        config = {}
-                        config['interfaces'] = []
+                        config = decorative_node.get_config()
                         for connection in connections:
                             (device, remote_interface, local_interface) = connection.split(':', 2)
                             self.decorative_node_connections[(device, remote_interface, local_interface)] = decorative_node.id
-                            config['interfaces'].append(local_interface)
-                        decorative_node.set_config(config)
+                            if local_interface not in config['interfaces']:
+                                config['interfaces'].append(local_interface)
 
         # update next ID for cloud
         if max_cloud_id != -1:
@@ -427,6 +487,7 @@ class NETFile(object):
             splash.show()
             splash.showMessage(translate("NETFile", "Please wait while importing the topology"))
             self.dynagen.ghosting()
+            self.dynagen.jitsharing()
         except lib.DynamipsError, msg:
             QtGui.QMessageBox.critical(globals.GApp.mainWindow, translate("NETFile", "Dynamips error"),  unicode(msg))
             globals.GApp.workspace.projectFile = None
@@ -462,7 +523,7 @@ class NETFile(object):
         max_frsw_id = -1
         max_atmsw_id = -1
         max_atmbr_id = -1
-        max_fw_id = -1
+        max_emu_id = -1
         for (devicename, device) in  self.dynagen.devices.iteritems():
 
             if isinstance(device,  lib.Bridge):
@@ -603,19 +664,19 @@ class NETFile(object):
                     if id > max_atmbr_id:
                         max_atmbr_id = id
 
-            elif isinstance(device, pix.FW):
+            elif isinstance(device, qlib.AnyEmuDevice):
 
-                node = self.create_node(device, 'PIX firewall', 'FW ' + device.name)
+                node = self.create_node(device, device._ufd_machine, device.gen_cfg_name())
                 assert(node)
                 node.set_hypervisor(device.dynamips)
                 self.configure_node(node, device)
                 node.create_config()
-                self.populate_connection_list_for_fw(device, connection_list)
-                match_obj = firewall_hostname_re.match(node.hostname)
+                self.populate_connection_list_for_emulated_device(device, connection_list)
+                match_obj = emu_hostname_re.match(node.hostname)
                 if match_obj:
                     id = int(match_obj.group(1))
-                    if id > max_fw_id:
-                        max_fw_id = id
+                    if id > max_emu_id:
+                        max_emu_id = id
 
             globals.GApp.topology.addItem(node)
 
@@ -630,8 +691,8 @@ class NETFile(object):
             init_atmsw_id(max_atmsw_id + 1)
         if max_atmbr_id != -1:
             init_atmbr_id(max_atmbr_id + 1)
-        if max_fw_id != -1:
-            init_fw_id(max_fw_id + 1)
+        if max_emu_id != -1:
+            init_emu_id(max_emu_id + 1)
 
         # update current hypervisor base port and base UDP
         base_udp = 0
@@ -682,12 +743,12 @@ class NETFile(object):
             config = config.replace('\r', "")
             # Write out the config to a file
             file_path = os.path.normpath(globals.GApp.workspace.projectConfigs) + os.sep + device.name + '.cfg'
-            print unicode(translate("NETFile", "Exporting %s configuration to %s")) % (device.name, file_path)
+            debug(unicode(translate("NETFile", "Exporting %s configuration to %s")) % (device.name, file_path))
         except lib.DynamipsError, msg:
-            print unicode(device.name + ': ' + translate("NETFile", "Dynamips error") + ': ') + unicode(msg)
+            debug(unicode(device.name + ': ' + translate("NETFile", "Dynamips error") + ': ') + unicode(msg))
             return
         except lib.DynamipsWarning, msg:
-            print unicode(device.name + ': ' + translate("NETFile", "Dynamips warning") + ': ') + unicode(msg)
+            debug(unicode(device.name + ': ' + translate("NETFile", "Dynamips warning") + ': ') + unicode(msg))
             return
         except:
             error('Unknown error exporting config for ' + device.name)
@@ -711,7 +772,13 @@ class NETFile(object):
         hypervisors = self.dynagen.dynamips.copy()
         for (name, hypervisor) in hypervisors.iteritems():
             if isinstance(hypervisor, lib.Dynamips) and len(hypervisor.devices) == 0:
-                del self.dynagen.dynamips[name]
+                has_ethsw = False
+                for item in globals.GApp.topology.items():
+                    if isinstance(item, ETHSW) and item.hypervisor and item.hypervisor == hypervisor:
+                        has_ethsw = True
+                        break
+                if not has_ethsw:
+                    del self.dynagen.dynamips[name]
 
         for hypervisor in self.dynagen.dynamips.values():
             hypervisor.configchange = True
@@ -725,6 +792,7 @@ class NETFile(object):
                 self.export_router_config(device)
 
         note_nb = 1
+        shape_nb = 1
         pix_nb = 1
         for item in globals.GApp.topology.items():
             # record clouds
@@ -744,26 +812,76 @@ class NETFile(object):
                 if item.hostname_xpos and item.hostname_ypos:
                     config['hx'] = item.hostname_xpos
                     config['hy'] = item.hostname_ypos
+                zvalue = item.zValue()
+                if zvalue != 0:
+                    config['z'] = zvalue
                 # record connections
                 connections = ''
                 for interface in item.getConnectedInterfaceList():
                     neighbor = item.getConnectedNeighbor(interface)
                     connections = connections + neighbor[0].hostname + ':' + neighbor[1] + ':' + interface + ' '
                 if connections:
-                    config['connections'] = connections
+                    config['connections'] = connections.strip()
             # record notes
-            elif isinstance(item, Annotation):
+            elif isinstance(item, Annotation) and item.autoGenerated == False:
                 if not self.dynagen.running_config.has_key('GNS3-DATA'):
                     self.dynagen.running_config['GNS3-DATA'] = {}
                 self.dynagen.running_config['GNS3-DATA']['NOTE ' + str(note_nb)] = {}
                 config = self.dynagen.running_config['GNS3-DATA']['NOTE ' + str(note_nb)] 
+
                 config['text'] = unicode(item.toPlainText()).replace("\n", "\\n")
                 config['x'] = item.x()
                 config['y'] = item.y()
+                
+                if item.deviceName and item.deviceIf:
+                    config['interface'] = item.deviceName + ' ' + item.deviceIf
+                    
+                if item.font() != QtGui.QFont("TypeWriter", 10, QtGui.QFont.Bold):
+                    config['font'] = '"' + str(item.font().toString()) + '"'
+                if item.rotation != 0:
+                    config['rotate'] = item.rotation
+                if item.defaultTextColor() != QtCore.Qt.black:
+                    config['color'] = '"' + str(item.defaultTextColor().name()) + '"'
+
                 zvalue = item.zValue()
-                if zvalue > 0:
+                if zvalue != 1:
                     config['z'] = zvalue
                 note_nb += 1
+                
+            # record shape items
+            elif isinstance(item, AbstractShapeItem):
+                if not self.dynagen.running_config.has_key('GNS3-DATA'):
+                    self.dynagen.running_config['GNS3-DATA'] = {}
+                self.dynagen.running_config['GNS3-DATA']['SHAPE ' + str(shape_nb)] = {}
+                config = self.dynagen.running_config['GNS3-DATA']['SHAPE ' + str(shape_nb)] 
+                if isinstance(item, QtGui.QGraphicsRectItem):
+                    config['type'] = 'rectangle'
+                else:
+                    config['type'] = 'ellipse'
+
+                config['x'] = item.x()
+                config['y'] = item.y()
+                rect = item.rect()
+                config['width'] = rect.width()
+                config['height'] = rect.height()
+
+                brush = item.brush()
+                if brush.style() != QtCore.Qt.NoBrush and brush.color() != QtCore.Qt.transparent:
+                    config['fill_color'] = '"' + str(brush.color().name()) + '"'
+                if item.rotation != 0:
+                    config['rotate'] = item.rotation
+                pen = item.pen()
+                if pen.color() != QtCore.Qt.black:
+                    config['border_color'] = '"' + str(pen.color().name()) + '"'
+                if pen.width() != 2:
+                    config['border_width'] = pen.width()
+                if pen.style() != QtCore.Qt.SolidLine:
+                    config['border_style'] = pen.style()
+                zvalue = item.zValue()
+                if zvalue != 0:
+                    config['z'] = zvalue
+                shape_nb += 1
+
             # record inserted images
             elif isinstance(item, Pixmap):
                 if not self.dynagen.running_config.has_key('GNS3-DATA'):
@@ -794,7 +912,7 @@ class NETFile(object):
                     neighbor = item.getConnectedNeighbor(interface)
                     connections = connections + neighbor[0].hostname + ':' + neighbor[1] + ':' + interface + ' '
                 if connections:
-                    config['connections'] = connections
+                    config['connections'] = connections.strip()
             elif isinstance(item, AbstractNode):
                 if globals.GApp.workspace.flg_showHostname:
                     # ugly but simple method to force to record hostname x&y positions
@@ -802,16 +920,22 @@ class NETFile(object):
                     item.showHostname()
                 # record node x & y positions
                 if not item.d:
-                    print item.hostname + unicode(' ' + translate("NETFile", "must be connected in order to be registered"))
+                    print item.hostname + unicode(' ' + translate("NETFile", "must be connected or have a hypervisor set in order to be registered"))
                     continue
                 if not item.default_symbol:
                     self.dynagen.running_config[item.d][item.get_running_config_name()]['symbol'] = item.type
-                self.dynagen.running_config[item.d][item.get_running_config_name()]['x'] = item.x()
-                self.dynagen.running_config[item.d][item.get_running_config_name()]['y'] = item.y()
-                # record hostname x & y positions
-                if item.hostname_xpos and item.hostname_ypos:
-                    self.dynagen.running_config[item.d][item.get_running_config_name()]['hx'] = item.hostname_xpos
-                    self.dynagen.running_config[item.d][item.get_running_config_name()]['hy'] = item.hostname_ypos
+                try:          
+                    self.dynagen.running_config[item.d][item.get_running_config_name()]['x'] = item.x()
+                    self.dynagen.running_config[item.d][item.get_running_config_name()]['y'] = item.y()
+                    zvalue = item.zValue()
+                    if zvalue != 0:
+                        self.dynagen.running_config[item.d][item.get_running_config_name()]['z'] = zvalue
+                    # record hostname x & y positions
+                    if item.hostname_xpos and item.hostname_ypos: #and \
+                        self.dynagen.running_config[item.d][item.get_running_config_name()]['hx'] = item.hostname_xpos
+                        self.dynagen.running_config[item.d][item.get_running_config_name()]['hy'] = item.hostname_ypos
+                except:
+                    pass
 
         # record project settings
         if globals.GApp.workspace.projectConfigs or globals.GApp.workspace.projectWorkdir:
@@ -819,15 +943,9 @@ class NETFile(object):
                 self.dynagen.running_config['GNS3-DATA'] = {}
             config = self.dynagen.running_config['GNS3-DATA']
             if globals.GApp.workspace.projectConfigs:
-                try:
-                    config['configs'] = relpath(globals.GApp.workspace.projectConfigs, os.path.dirname(path))
-                except OSError:
-                    config['configs'] = globals.GApp.workspace.projectConfigs
+                config['configs'] = self.convert_to_relpath(globals.GApp.workspace.projectConfigs, path)
             if globals.GApp.workspace.projectWorkdir:
-                try:
-                    config['workdir'] = relpath(globals.GApp.workspace.projectWorkdir, os.path.dirname(path))
-                except OSError:
-                    config['workdir'] = globals.GApp.workspace.projectWorkdir
+                config['workdir'] = self.convert_to_relpath(globals.GApp.workspace.projectWorkdir, path)
 
         # register matrix data
         matrix = globals.GApp.scene.matrix()
@@ -839,6 +957,65 @@ class NETFile(object):
             self.dynagen.running_config['GNS3-DATA']['m11'] = m11
             self.dynagen.running_config['GNS3-DATA']['m22'] = m22
 
+        # register scene size
+        scene_width = int(globals.GApp.topology.width())
+        scene_height = int(globals.GApp.topology.height())
+        if scene_width != 2000 or scene_height != 1000:
+            if not self.dynagen.running_config.has_key('GNS3-DATA'):
+                self.dynagen.running_config['GNS3-DATA'] = {}
+            self.dynagen.running_config['GNS3-DATA']['width'] = scene_width
+            self.dynagen.running_config['GNS3-DATA']['height'] = scene_height
+
+        # autostart
+        autostart = False
+        for (name, val) in self.dynagen.autostart.iteritems():
+            if val == True:
+                autostart = True
+                break
+        self.dynagen.running_config['autostart'] = autostart
+        
+        if globals.GApp.systconf['general'].relative_paths:
+            # Change absolute paths to relative paths if same base as the config file
+            for hypervisor in self.dynagen.dynamips.values():
+                if isinstance(hypervisor, qlib.Qemu):
+                    h = 'qemu ' + hypervisor.host + ":" + str(hypervisor.port)          
+                else:
+                    h = hypervisor.host + ":" + str(hypervisor.port)
+                config = self.dynagen.running_config[h]
+                if config.has_key('workingdir'):
+                    config['workingdir'] = self.convert_to_relpath(config['workingdir'], path)
+    
+                for model in dynagen_namespace.DEVICETUPLE:
+                    if config.has_key(model):
+                        # ASA has no image
+                        if model == '5520':
+                            config[model]['initrd'] = self.convert_to_relpath(config[model]['initrd'], path)
+                            config[model]['kernel'] = self.convert_to_relpath(config[model]['kernel'], path)
+                        # IDS-4215 has no default image
+                        elif model == 'IDS-4215':
+                            config[model]['image1'] = self.convert_to_relpath(config[model]['image1'], path)
+                            config[model]['image2'] = self.convert_to_relpath(config[model]['image2'], path)
+                        else:
+                            config[model]['image'] = self.convert_to_relpath(config[model]['image'], path)
+         
+                for subsection in config.sections:
+                    device = config[subsection]
+                    if device.has_key('cnfg') and device['cnfg']:
+                        device['cnfg'] = self.convert_to_relpath(device['cnfg'], path)
+        
         self.dynagen.running_config.filename = path
         self.dynagen.running_config.write()
         self.dynagen.running_config.filename = None
+        
+    def convert_to_relpath(self, path, config_path):
+        """ Returns a relative path when the config path and another path share a common base directory
+        """
+
+        real_image_path = os.path.realpath(path)
+        config_dir = os.path.dirname(os.path.realpath(config_path))
+        commonprefix = os.path.commonprefix([real_image_path, config_dir])
+        if config_dir == commonprefix:
+            relpath = os.path.relpath(real_image_path, commonprefix)
+            debug("Convert path " + path + " to a relative path : " + relpath)
+            return relpath
+        return path
