@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # vim: expandtab ts=4 sw=4 sts=4:
 #
-# Copyright (C) 2007 GNS-3 Dev Team
+# Copyright (C) 2007-2010 GNS3 Development Team (http://www.gns3.net/team).
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License version 2 as
@@ -16,10 +16,11 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 #
-# Contact: contact@gns3.net
+# code@gns3.net
 #
 
-import sys, os
+import sys
+import os
 import GNS3.Globals as globals
 from PyQt4 import QtGui, QtCore
 from GNS3.Config.Objects import systemGeneralConf
@@ -44,8 +45,25 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
         self.connect(self.pushButton_ClearConfiguration, QtCore.SIGNAL('clicked()'), self.__clearConfiguration)
         self.connect(self.ProjectPath_browser, QtCore.SIGNAL('clicked()'), self.__setProjectPath)
         self.connect(self.IOSPath_browser, QtCore.SIGNAL('clicked()'), self.__setIOSPath)
+        self.connect(self.pushButtonUseTerminalCommand, QtCore.SIGNAL('clicked()'), self.__setTerminalCmd)
         self.loadConf()
 
+        terminal_cmds = {'Putty (Windows 64 bits)': 'C:\Program Files (x86)\Putty\putty.exe -telnet %h %p',
+                         'Putty (Windows 32 bits)': 'C:\Program Files\Putty\putty.exe -telnet %h %p',
+                         'Putty (Windows, included with GNS3)': 'putty.exe -telnet %h %p',
+                         'SecureCRT (Windows 32 bits)': 'start C:\progra~1\vandyk~1\SecureCRT\SecureCRT.EXE /script C:\progra~1\gns3\securecrt.vbs /arg %d /T /telnet %h %p',
+                         'SecureCRT (Windows 64 bits)': 'start C:\progra~2\vandyk~1\SecureCRT\SecureCRT.EXE /script C:\progra~2\gns3\securecrt.vbs /arg %d /T /telnet %h %p',
+                         'TeraTerm (Windows)': 'C:\TTERMPRO\\ttssh.exe %h %p /W=%d /T=1',
+                         'Telnet (Windows)': 'start telnet %h %p',
+                         'xterm (Linux)': 'xterm -T %d -e \'telnet %h %p\' >/dev/null 2>&1 &',
+                         'Kconsole (Linux KDE)': '/usr/lib/kde4/bin/konsole --new-tab -p tabtitle=%d -e telnet %h %p >/dev/null 2>&1 &',
+                         'Terminal (Mac OS X)': "/usr/bin/osascript -e 'tell application \"terminal\" to do script with command \"telnet %h %p ; exit\"'",
+                         'iTerm (Mac OS X)': "/usr/bin/osascript -e 'tell app \"iTerm\"' -e 'activate' -e 'set myterm to the first terminal' -e 'tell myterm' -e 'set mysession to (make new session at the end of sessions)' -e 'tell mysession' -e 'exec command \"telnet %h %p\"' -e 'set name to \"%d\"' -e 'end tell' -e 'end tell' -e 'end tell'"
+                         }
+        
+        for (name, cmd) in terminal_cmds.iteritems():
+            self.comboBoxPreconfigTerminalCommands.addItem(name, cmd)
+
     def loadConf(self):
 
         if globals.GApp.systconf.has_key('general'):
@@ -62,18 +80,16 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
                 self.langsBox.setCurrentIndex(idx)
             idx += 1
                 
+        # Slow start when starting every devices
+        self.slowStartAll.setValue(self.conf.slow_start)
+                
         # Defaults terminal command
         if self.conf.term_cmd == '':
             if sys.platform.startswith('darwin'):
                 self.conf.term_cmd = unicode("/usr/bin/osascript -e 'tell application \"terminal\" to do script with command \"telnet %h %p ; exit\"'")
             elif sys.platform.startswith('win'):
-                if os.path.lexists('C:\\WINDOWS\\system32\\telnet.exe'):
-                    # check if telnet is there
-                    self.conf.term_cmd = unicode("start telnet %h %p")
-                else:
-                    # else try to use putty
-                    self.conf.term_cmd = unicode('C:\Program Files\Putty\putty.exe -telnet %h %p')
-                    self.conf.use_shell = False
+                self.conf.term_cmd = unicode('putty.exe -telnet %h %p')
+                self.conf.use_shell = False
             else:
                 self.conf.term_cmd = unicode("xterm -T %d -e 'telnet %h %p' >/dev/null 2>&1 &")
 
@@ -83,7 +99,7 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
             elif os.environ.has_key("TMP"):
                 self.conf.project_path = unicode(os.environ["TMP"],  errors='replace')
             else:
-                self.conf.project_path = unicode('/tmp')
+                self.conf.project_path = unicode('/tmp', errors='replace')
 
         if self.conf.ios_path == '':
             if os.environ.has_key("TEMP"):
@@ -91,15 +107,16 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
             elif os.environ.has_key("TMP"):
                 self.conf.ios_path = unicode(os.environ["TMP"],  errors='replace')
             else:
-                self.conf.ios_path = unicode('/tmp')
+                self.conf.ios_path = unicode('/tmp', errors='replace')
 
         self.lineEditTermCommand.setText(self.conf.term_cmd)
         self.ProjectPath.setText(os.path.normpath(self.conf.project_path))
         self.IOSPath.setText(os.path.normpath(self.conf.ios_path))
+
+        # GUI settings        
         self.workspaceWidth.setValue(self.conf.scene_width)
         self.workspaceHeight.setValue(self.conf.scene_height)
-            
-        # GUI settings
+        
         if self.conf.status_points == True:
             self.checkBoxShowStatusPoints.setCheckState(QtCore.Qt.Checked)
         else:
@@ -122,7 +139,12 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
         else:
             self.checkBoxDrawRectangle.setCheckState(QtCore.Qt.Unchecked)
             
-        self.labelConfigurationPath.setText(ConfDB().fileName())
+        if self.conf.relative_paths == True:
+            self.checkBoxRelativePaths.setCheckState(QtCore.Qt.Checked)
+        else:
+            self.checkBoxRelativePaths.setCheckState(QtCore.Qt.Unchecked)
+            
+        self.labelConfigurationPath.setText(os.path.normpath(unicode(ConfDB().fileName())))
 
     def saveConf(self):
 
@@ -161,11 +183,17 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
         else:
             self.conf.draw_selected_rectangle = False
 
+        if self.checkBoxRelativePaths.checkState() == QtCore.Qt.Checked:
+            self.conf.relative_paths = True
+        else:
+            self.conf.relative_paths = False
+
         self.conf.project_path = unicode(self.ProjectPath.text())
         self.conf.ios_path = unicode(self.IOSPath.text())
         self.conf.term_cmd = unicode(self.lineEditTermCommand.text())
         self.conf.scene_width = self.workspaceWidth.value()
         self.conf.scene_height = self.workspaceHeight.value()
+        self.conf.slow_start = self.slowStartAll.value()
         
         # Apply scene size
         globals.GApp.topology.setSceneRect(-(self.conf.scene_width / 2), -(self.conf.scene_height / 2), self.conf.scene_width, self.conf.scene_height)
@@ -188,6 +216,12 @@ class UiConfig_PreferencesGeneral(QtGui.QWidget, Ui_PreferencesGeneral):
 
         if path:
             self.IOSPath.setText(os.path.normpath(path))
+            
+    def __setTerminalCmd(self):
+    
+        self.lineEditTermCommand.clear()
+        command = self.comboBoxPreconfigTerminalCommands.itemData(self.comboBoxPreconfigTerminalCommands.currentIndex(), QtCore.Qt.UserRole).toString()
+        self.lineEditTermCommand.setText(command)
 
     def __clearConfiguration(self):
     
