@@ -7,29 +7,32 @@ FILES=" Form_MainWindow
         Form_NodeConfigurator
 	Form_PreferencesDialog
 	Form_NewProject
+	Form_Snapshots
 	Form_SymbolManager
 	Form_Wizard
 	Form_StyleDialog
+	Form_StartupConfig
         ./ConfigurationPages/Form_IOSRouterPage
         ./ConfigurationPages/Form_CloudPage
         ./ConfigurationPages/Form_ETHSWPage
         ./ConfigurationPages/Form_FRSWPage
-        ./ConfigurationPages/Form_HubPage
         ./ConfigurationPages/Form_ATMSWPage
         ./ConfigurationPages/Form_ATMBRPage
         ./ConfigurationPages/Form_FWPage
-        ./ConfigurationPages/Form_SIMHOSTPage
+        ./ConfigurationPages/Form_ASAPage
+        ./ConfigurationPages/Form_JunOSPage
+        ./ConfigurationPages/Form_IDSPage
+        ./ConfigurationPages/Form_QemuPage
         ./ConfigurationPages/Form_DecorativeNodePage
         ./ConfigurationPages/Form_PreferencesGeneral
         ./ConfigurationPages/Form_PreferencesDynamips
         ./ConfigurationPages/Form_PreferencesCapture
-        ./ConfigurationPages/Form_PreferencesPemu
-        ./ConfigurationPages/Form_PreferencesSimhost
+        ./ConfigurationPages/Form_PreferencesQemu
 "
 
 # Update files...
 for file in $FILES;
 do
     echo "Generating $file"
-    pyuic4 "$file.ui" > "$file.py"
+    /opt/local/var/macports/software/py26-pyqt4/4.6.2_0/opt/local/bin/pyuic4-2.6 "$file.ui" > "$file.py"
 done
